//
//  CoreDataAppDelegate.m
//  CoreData
//
//  Created by NYU User on 5/6/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "CoreDataAppDelegate.h"
#import "Person.h"

@implementation CoreDataAppDelegate
@synthesize window;

#pragma mark -
#pragma mark Application lifecycle

- (BOOL) application: (UIApplication *) application
	didFinishLaunchingWithOptions: (NSDictionary *) launchOptions {    

	// Override point for customization after application launch.
	NSLog(@"applicationDocumentsDirectory == %@",
		[self applicationDocumentsDirectory]);

	UIScreen *s = [UIScreen mainScreen];
	textView = [[UITextView alloc] initWithFrame: s.applicationFrame];
	textView.font = [UIFont systemFontOfSize: 20];

	window = [[UIWindow alloc] initWithFrame: s.bounds];
	[window addSubview: textView];
	[window makeKeyAndVisible];

	//Create John Doe.

	Person *johnDoe = [NSEntityDescription insertNewObjectForEntityForName: @"Person"
		inManagedObjectContext: self.managedObjectContext
	];

	johnDoe.lastname = @"Doe";
	johnDoe.firstname = @"John";
	johnDoe.ss = [NSNumber numberWithInt: 123456789];

	//Write John Doe into the database.

	NSError *error = nil;
	if (![self.managedObjectContext save: &error]) {
		// Handle the error.
		NSLog(@"save: failed: %@", error);
		[error release];
	}

	//Create Jane Doe.

	Person *janeDoe = [NSEntityDescription insertNewObjectForEntityForName: @"Person"
		inManagedObjectContext: self.managedObjectContext
	];

	//Write Jane Doe into the database.

	janeDoe.lastname = @"Doe";
	janeDoe.firstname = @"Jane";
	janeDoe.ss = [NSNumber numberWithInt: 987654321];

	if (![self.managedObjectContext save: &error]) {
		// Handle the error.
		NSLog(@"save: failed: %@", error);
		[error release];
	}

	//Read all the Person records from the database
	//and display them in the UITextView.

	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	NSEntityDescription *entityDescription = [NSEntityDescription entityForName: @"Person"
		inManagedObjectContext: self.managedObjectContext
	];
	[fetchRequest setEntity: entityDescription];

	error = nil;
	NSArray *results =
		[self.managedObjectContext executeFetchRequest: fetchRequest error: &error];
	if (results == nil) {
		// Handle the error.
		NSLog(@"executeFetchRequest:error: failed: %@", error);
		[error release];
	}
	[fetchRequest release];

	textView.text = [textView.text stringByAppendingString: @"Unsorted:\n"];
	for (Person *person in results) { 
		textView.text = [textView.text stringByAppendingFormat: @"%@\n", person];
	}
	textView.text = [textView.text stringByAppendingString: @"\n"];
	
	//Read all the Person records from the database in order of last name,
	//firstname, and ss.  Display the records in the UITextView.
	
	fetchRequest = [[NSFetchRequest alloc] init];
	entityDescription = [NSEntityDescription entityForName: @"Person"
		inManagedObjectContext: self.managedObjectContext
	];
	[fetchRequest setEntity: entityDescription];

	NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:
		[[NSSortDescriptor alloc] initWithKey: @"lastname" ascending: YES],
		[[NSSortDescriptor alloc] initWithKey: @"firstname" ascending: YES],
		[[NSSortDescriptor alloc] initWithKey: @"ss" ascending: YES],
		nil
	];

	[fetchRequest setSortDescriptors: sortDescriptors];

	for (NSSortDescriptor *sortDescriptor in sortDescriptors) {
		[sortDescriptor release];
	}
	[sortDescriptors release];
	
	error = nil;
	results =
		[self.managedObjectContext executeFetchRequest: fetchRequest error: &error];
	if (results == nil) {
		// Handle the error.
		NSLog(@"executeFetchRequest:error: failed: %@", error);
		[error release];
	}
	[fetchRequest release];

	textView.text = [textView.text stringByAppendingString: @"Sorted:\n"];
	for (Person *person in results) { 
		textView.text = [textView.text stringByAppendingFormat: @"%@\n", person];
	}
	textView.text = [textView.text stringByAppendingString: @"\n"];
	
	//Read all the Person records whose first name is "John". 
	//Display the record(s) in the UITextView.
	
	fetchRequest = [[NSFetchRequest alloc] init];
	entityDescription = [NSEntityDescription entityForName: @"Person"
		inManagedObjectContext: self.managedObjectContext
	];
	[fetchRequest setEntity: entityDescription];
	
	NSPredicate *predicate =
		[NSPredicate predicateWithFormat: @"firstname == 'John'"];;
	[fetchRequest setPredicate: predicate];
	
	error = nil;
	results =
	[self.managedObjectContext executeFetchRequest: fetchRequest error: &error];
	if (results == nil) {
		// Handle the error.
		NSLog(@"executeFetchRequest:error: failed: %@", error);
		[error release];
	}
	[fetchRequest release];

	textView.text = [textView.text stringByAppendingString: @"Just John:\n"];
	for (Person *person in results) { 
		textView.text = [textView.text stringByAppendingFormat: @"%@\n", person];
	}
	textView.text = [textView.text stringByAppendingString: @"\n"];

	//Remove John Doe from the database.

	[self.managedObjectContext deleteObject: johnDoe];
	// Commit the change.
	error = nil;
	if (![self.managedObjectContext save: &error]) {
		// Handle the error.
		NSLog(@"save: failed: %@", error);
		[error release];
	}

	//Read all the Person records from the database
	//and display them in the UITextView.
	
	fetchRequest = [[NSFetchRequest alloc] init];
	entityDescription = [NSEntityDescription entityForName: @"Person"
		inManagedObjectContext: self.managedObjectContext
	];
	[fetchRequest setEntity: entityDescription];
	
	error = nil;
	results =
		[self.managedObjectContext executeFetchRequest: fetchRequest error: &error];
	if (results == nil) {
		// Handle the error.
		NSLog(@"executeFetchRequest:error: failed: %@", error);
		[error release];
	}
	[fetchRequest release];

	textView.text = [textView.text stringByAppendingString: @"Delete John:\n"];
	for (Person *person in results) { 
		textView.text = [textView.text stringByAppendingFormat: @"%@\n", person];
	}

	return YES;
}


- (void) applicationWillResignActive: (UIApplication *) application {
    /*
     Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
     Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
     */
}


- (void) applicationDidEnterBackground: (UIApplication *) application {
    /*
     Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
     If your application supports background execution, called instead of applicationWillTerminate: when the user quits.
     */
    [self saveContext];
}


- (void) applicationWillEnterForeground: (UIApplication *) application {
    /*
     Called as part of the transition from the background to the inactive state: here you can undo many of the changes made on entering the background.
     */
}


- (void) applicationDidBecomeActive: (UIApplication *) application {
    /*
     Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
     */
}


/**
 applicationWillTerminate: saves changes in the application's managed object context before the application terminates.
 */
- (void) applicationWillTerminate: (UIApplication *) application {
    [self saveContext];
}


- (void) saveContext {
    
    NSError *error = nil;
    if (managedObjectContext_ != nil) {
        if ([managedObjectContext_ hasChanges] && ![managedObjectContext_ save:&error]) {
            /*
             Replace this implementation with code to handle the error appropriately.
             
             abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
             */
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            abort();
        } 
    }
}    


#pragma mark -
#pragma mark Core Data stack

/**
 Returns the managed object context for the application.
 If the context doesn't already exist, it is created and bound to the persistent store coordinator for the application.
 */
- (NSManagedObjectContext *) managedObjectContext {
	if (managedObjectContext_ == nil) {
		NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
		if (coordinator != nil) {
			managedObjectContext_ = [[NSManagedObjectContext alloc] init];
			[managedObjectContext_ setPersistentStoreCoordinator: coordinator];
		
		}
	}
	return managedObjectContext_;
}


/**
 Returns the managed object model for the application.
 If the model doesn't already exist, it is created from the application's model.
 */
- (NSManagedObjectModel *) managedObjectModel {

	if (managedObjectModel_ == nil) {
		NSString *modelPath =
			[[NSBundle mainBundle] pathForResource: @"CoreData" ofType: @"momd"];
		NSURL *modelURL = [NSURL fileURLWithPath: modelPath];
		managedObjectModel_ =
			[[NSManagedObjectModel alloc] initWithContentsOfURL: modelURL];
	}
	return managedObjectModel_;
}


/**
 Returns the persistent store coordinator for the application.
 If the coordinator doesn't already exist, it is created and the application's store added to it.
 */
- (NSPersistentStoreCoordinator *) persistentStoreCoordinator {
    
    if (persistentStoreCoordinator_ != nil) {
        return persistentStoreCoordinator_;
    }
    
    NSURL *storeURL = [NSURL fileURLWithPath: [[self applicationDocumentsDirectory] stringByAppendingPathComponent: @"CoreData.sqlite"]];
    
    NSError *error = nil;
    persistentStoreCoordinator_ = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:[self managedObjectModel]];
    if (![persistentStoreCoordinator_ addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:storeURL options:nil error:&error]) {
        /*
         Replace this implementation with code to handle the error appropriately.
         
         abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
         
         Typical reasons for an error here include:
         * The persistent store is not accessible;
         * The schema for the persistent store is incompatible with current managed object model.
         Check the error message to determine what the actual problem was.
         
         
         If the persistent store is not accessible, there is typically something wrong with the file path. Often, a file URL is pointing into the application's resources directory instead of a writeable directory.
         
         If you encounter schema incompatibility errors during development, you can reduce their frequency by:
         * Simply deleting the existing store:
         [[NSFileManager defaultManager] removeItemAtURL:storeURL error:nil]
         
         * Performing automatic lightweight migration by passing the following dictionary as the options parameter: 
         [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],NSMigratePersistentStoresAutomaticallyOption, [NSNumber numberWithBool:YES], NSInferMappingModelAutomaticallyOption, nil];
         
         Lightweight migration will only work for a limited set of schema changes; consult "Core Data Model Versioning and Data Migration Programming Guide" for details.
         
         */
        NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        abort();
    }    
    
    return persistentStoreCoordinator_;
}


#pragma mark -
#pragma mark Application's Documents directory

/**
 Returns the path to the application's Documents directory.
 */
- (NSString *) applicationDocumentsDirectory {
    return [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) lastObject];
}


#pragma mark -
#pragma mark Memory management

- (void) applicationDidReceiveMemoryWarning: (UIApplication *) application {
    /*
     Free up as much memory as possible by purging cached data objects that can be recreated (or reloaded from disk) later.
     */
}


- (void) dealloc {
	[managedObjectContext_ release];
	[managedObjectModel_ release];
	[persistentStoreCoordinator_ release];
	
	[window release];
	[textView release];
	[super dealloc];
}

@end
